<?php
/*
Copyright 2011 da UFRGS - Universidade Federal do Rio Grande do Sul

Este arquivo  parte do programa SAELE - Sistema Aberto de Eleies Eletrnicas.

O SAELE  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos
termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre
(FSF); na verso 2 da Licena.

Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA;
sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR.
Veja a Licena Pblica Geral GNU/GPL em portugus para maiores detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt",
junto com este programa, se no, acesse o Portal do Software Pblico Brasileiro no
endereo www.softwarepublico.gov.br ou escreva para a Fundao do Software Livre(FSF)
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
*/

final class Controlador {
    private $PessoaLogada = NULL;
    private $ConcursoEdicao = NULL;
    private $EleicaoEdicao = NULL;
    private $ConcursoVotacao = NULL;
    private $EleicaoVotacao = NULL;
    private $VetorCedula = array();
    private $NrVotoAtual = NULL;
    private $Origem = NULL;

    private function __construct(PessoaEleicao $Pessoa) {
        $this->PessoaLogada = $Pessoa;
    }

/**
 *
 * @param int $CodPessoa
 * @return Controlador
 */
    public static function instancia($Pessoa = null) {
        if(isset($_SESSION['Controlador']))
            return $_SESSION['Controlador'];
        elseif(!is_null($Pessoa)) {
            if($Pessoa->valido()) {
                $_SESSION['Controlador'] = new Controlador($Pessoa);
                return $_SESSION['Controlador'];
            }
            else throw new ControladorException("Pessoa invlida", 0);
        }
        else {
            ob_clean();
            echo "Sesso expirada";
            exit;
        }
    }

/**
 *
 * @return PessoaEleicao
 */
    public function recuperaPessoaLogada() {
        return $this->PessoaLogada;
    }

    public function registraEleicaoEdicao(ConcursoEleitoral $Concurso, Eleicao $Eleicao) {
        if($this->PessoaLogada->eGerenteSistema() || ($Eleicao->verificaComissao($this->PessoaLogada) == COMISSAO_GERENTE)) {
            $this->ConcursoEdicao = $Concurso;
            $this->EleicaoEdicao = $Eleicao;
            return true;
        }
        else throw new ControladorException("Permisso negada", 0);
    }

    public function removeEleicaoEdicao() {
        unset($this->ConcursoEdicao);
        $this->ConcursoEdicao = null;
        unset($this->EleicaoEdicao);
        $this->EleicaoEdicao = null;
        return true;
    }

    public function registraConcursoVotacao(ConcursoEleitoral $Concurso) {
        if(!$Concurso->abertoParaVotacao()) {
            if($Concurso->estadoConcurso() < CONCURSO_INICIADO)
                throw new ControladorException("Concurso no iniciado", 0);
            if($Concurso->estadoConcurso() > CONCURSO_INICIADO)
                throw new ControladorException("Concurso j encerrado", 0);
        }
        $this->ConcursoVotacao = $Concurso;
        return true;
    }

    public function registraEleicaoVotacao(Eleicao $Eleicao) {
        if(!($this->ConcursoVotacao instanceof ConcursoEleitoral))
            throw new ControladorException("No h concurso registrado", 0);
        if($Eleicao->get("codconcurso") != $this->ConcursoVotacao->get("codconcurso"))
            throw new ControladorException("A Eleio no faz parte do Concurso", 0);
        $Eleitor = $Eleicao->devolveEleitor($this->PessoaLogada);
        if(is_null($Eleitor))
            throw new ControladorException("Voc no  eleitor desta eleio", 0);
        if(!is_null($Eleitor->get("datahoravoto")))
            throw new ControladorException("Voc j votou nesta eleio", 0);
        if($this->ConcursoVotacao->get("indbarradoporip") == "S") {
            $Urna = $Eleicao->devolveUrnaPorIP($_SERVER['REMOTE_ADDR']);
            if(is_null($Urna))
                throw new ControladorException("Mquina no autorizada nesta eleio", 0);
            if($Urna->get("indativa") != "S")
                throw new ControladorException("Mquina no autorizada nesta eleio", 0);
        }
        elseif($this->ConcursoVotacao->get("indbarradoporip") == "E") {
            $Escopo = $Eleicao->devolveEscopoPorPrefixoIP($_SERVER['REMOTE_ADDR']);
            if(is_null($Escopo))
                throw new ControladorException("Mquina no autorizada nesta eleio", 0);
            if($Escopo->get("indativa") != "S")
                throw new ControladorException("Mquina no autorizada nesta eleio", 0);
        }
        if(!$Eleicao->eleicaoZerada())
            throw new ControladorException("Eleio no iniciada", 0);
        $this->EleicaoVotacao = $Eleicao;
        return true;
    }

    public function removeEleicaoVotacao() {
        unset($this->EleicaoVotacao);
        $this->EleicaoVotacao = null;
    }
/**
 *
 * @return ConcursoEleitoral
 */
    public function recuperaConcursoEdicao() {
        if($this->ConcursoEdicao instanceof ConcursoEleitoral) {
            return $this->ConcursoEdicao;
        }
        else throw new ControladorException("No h concurso em edio", 1);
    }

/**
 *
 * @return ConcursoEleitoral
 */
    public function recuperaConcursoVotacao() {
        if($this->ConcursoVotacao instanceof ConcursoEleitoral) {
            $this->registraConcursoVotacao($this->ConcursoVotacao); // Refaz o registro para testar novamente as restries
            return $this->ConcursoVotacao;
        }
        else throw new ControladorException("No h concurso para votao", 1);
    }
/**
 *
 * @return Eleicao
 */
    public function recuperaEleicaoEdicao() {
        if($this->EleicaoEdicao instanceof Eleicao) {
            return $this->EleicaoEdicao;
        }
        else throw new ControladorException("No h eleio em edio", 2);
    }
/**
 *
 * @return Eleicao
 */
    public function recuperaEleicaoVotacao() {
        if($this->EleicaoVotacao instanceof Eleicao) {
            $this->registraEleicaoVotacao($this->EleicaoVotacao);
            return $this->EleicaoVotacao;
        }
        else throw new ControladorException("No h eleio para votao", 2);
    }

    public function inicializaVetorCedula() {
        $this->VetorCedula = array();
        array_fill(1, $this->EleicaoVotacao->get("nrpossibilidades"), "B");
    }

    public function registraNrVotoAtual($Voto) {
        if($Voto <= $this->EleicaoVotacao->get("nrpossibilidades"))
            $this->NrVotoAtual = $Voto;
        else throw new ControladorException("Voto alm do nmero de possibilidades", 3);
    }

    public function registraVoto($Voto) {
        if(($Voto == "B") || ($Voto == "N") || (!is_null($this->EleicaoVotacao->devolveChapaPorNumero($Voto)))) {
            if(is_numeric($Voto)) {
                $PosVoto = array_search($Voto, $this->VetorCedula);
                if(($PosVoto !== false) && ($PosVoto !== $this->NrVotoAtual))
                    throw new ControladorException("Voto repetido", 4);
            }
            $this->VetorCedula[$this->NrVotoAtual] = $Voto;
        }
        else throw new ControladorException("Voto invlido: ".$Voto, 5);
    }

    public function devolveVetorCedula() {
        return $this->VetorCedula;
    }

    public function devolveNrVotoAtual() {
        return $this->NrVotoAtual;
    }

    public function devolveVoto($Voto) {
        if(isset($this->VetorCedula[$Voto]))
            return $this->VetorCedula[$Voto];
    }

    public function registraOrigem($Origem) {
        $this->Origem = $Origem;
    }

    public function recuperaOrigem() {
        return $this->Origem;
    }


    public function __destruct() {
        $_SESSION['Controlador'] = $this;
    }
}

class ControladorException extends Exception {
}
?>
